<?php
/**
 * Class Icms
 *
 * main class of iCMS
 *
 * @access   public
 * @category Icms
 * @package  Icms
 * 
 * @access   public
 * @author   dark
 * @version  $Id: Icms.php Tue Nov 11 16:28:06 EET 2008 16:28:06 dark$
 */
require_once 'Icms/Request.php';

class Icms 
{
	/**
	 * Icms_Request instance
	 *
	 * @var Icms_Request
	 */
	public $Request;
	
	/**
	 * Icms_Abstract instance
	 *
	 * @var Icms_Abstract
	 */
	public $Abstract;
	
	/**
	 * Ftp wrapper class
	 *
	 * @var Icms_Ftp
	 */
	public $Ftp;
	
	protected $_placeholder = 'default.html';
	
	protected $_is_admin;
	protected $_is_main;
	protected $_is_dir;
	protected $_is_404;
	
	/**
	 * Icms options
	 *
	 * @var array
	 */
	protected $_options = array();
	
    /**
     * Constructor of Icms
     *
     * @access  public
     * @param   array $options
     */
    function __construct($options) 
    {
    	if (strpos($options[ICMS_ADMIN], '.html') === false) {
    		$options[ICMS_ADMIN] .= '.html';
    	}
    	
        $this->_options = array_merge($this->_options, $options);
        
    }
    
    /**
     * init
     *
     * init Icms
     *
     * @access  public
     * @return  rettype  return
     */
    public function init() 
    {
        /**
         * Request Initialization
         */
        $this->Request = new Icms_Request();
        
        if (isset($_SESSION['admin']) && $_SESSION['admin']) {
            $this->isAdmin(true);            
        }
        
        if ($this->Request->isXmlHttpRequest() && $this->isAdmin()) {
            $this->ajax();
            return $this;
        }
        
        $file = $this->Request->getRequestUri();
        $path = ROOT . DS . $this->Request->getRequestUri();
        
        if ($file == 'index.html' or $file == 'index.php' or $file == Icms()->option(ICMS_PATH) or $file == '') {
            // ???
    		$this->isMain(true);
    	    // access to html files
    		$this->placeholder('index.html');
    	} elseif ($file == 'logout.html') {
    	    // admin logout
    		$this->isAdmin(false);
    		if (isset($_SESSION['placeholder'])) {    		    
    		    header('Location: '. $_SESSION['placeholder']);
    		} else {
    		    header('Location: index.html');
    		}
    		exit();
    	} elseif ($file == $this->option(ICMS_ADMIN)) {
    	    // admin login
    		$this->isAdmin(true);
    		if (isset($_SESSION['placeholder'])) {    		    
    		    header('Location: '. $_SESSION['placeholder']);
    		} else {
    		    header('Location: index.html');
    		}
    		exit();
    	} elseif ($this->isAdmin() && $this->Request->get('action') && $this->Request->get('action')=='upload') {
            require_once 'Icms/Admin.php';
            $this->Abstract = new Icms_Admin();
            $this->Abstract -> upload();
            exit();
            
        } elseif (empty($this->Request->info['extension'])) {
    	    // access to directory
    	    if (file_exists($path) && is_dir($path)) {
    	        $this->isDir(true);
    	    } elseif (file_exists($path . 'index.html')) {
    		    $this->placeholder($this->Request->uri . 'index.html');
    	    } else {
    	        $this->is404(true);
    	    }
    	} elseif ($this->Request->info['extension'] != 'html') {
    	    // access to not allowed files
    	    $this->is404(true);
    	} else {
    	    // access to html files
    		$this->placeholder($this->Request->uri);
    	}
    	
    	$this -> main();
    	
        return $this;
    }
    
    /**
     * main
     *
     * init Icms
     *
     * @access  public
     * @return  rettype  return
     */
    public function main() 
    {
        /**
         * Controller Initialization
         */
        if ($this->isAdmin()) {
            require_once 'Icms/Admin.php';
            $this->Abstract = new Icms_Admin();
        } else {
            require_once 'Icms/Main.php';
            $this->Abstract = new Icms_Main();
        }
        return $this;
    }
    
    /**
     * ajax
     *
     * init Icms
     *
     * @access  public
     * @return  rettype  return
     */
    public function ajax() 
    {
        /**
         * Controller Initialization
         */
        if ($this->isAdmin()) {
            require_once 'Icms/Admin/Ajax.php';
            $this->Abstract = new Icms_Admin_Ajax();
        } else {
            require_once 'Icms/Main/Ajax.php';
            $this->Abstract = new Icms_Main_Ajax();
        }
        return $this;
    }
    
    /**
     * render
     *
     * render 
     *
     * @access  public
     * @return  void
     */
    public function render() 
    {
        if ($this->Request->isXmlHttpRequest()) {
            $this->placeholder();
        } else {
            ob_start();
            if (isset($this->_options[ICMS_LAYOUT][$this->_placeholder]) && file_exists(ROOT .DS. 'include' .DS. $this->_options[ICMS_LAYOUT][$this->_placeholder])) {
                include_once ROOT .DS. 'include' .DS. $this->_options[ICMS_LAYOUT][$this->_placeholder];
            } else {
                include_once ROOT .DS. 'include' .DS. 'index.php';
            }
            ob_end_flush();
        }
    }
    
    /**
     * option
     *
     * return or set option
     *
     * @access  public
     * @param   string     $option
     * @param   string     $value
     * @return  Icms|mixed
     */
    public function option($option, $value = null) 
    {
        if ($value) {
        	$this->_options[$option] = $value;
        	return $this;
        } else {
        	return isset($this->_options[$option])?$this->_options[$option]:null;
        }
    }
    
    /**
     * placeholder
     *
     * placeholder renderer
     *
     * @access  public
     * @param   string   $value
     * @return  Icms
     */
    public function placeholder($value = null) 
    {
    	if (null !== $value) {
    		$this->_placeholder = $value;
    	} elseif ($this->isDir()) {
    	    $this->Abstract->directory();
	    } else {
    		$this->Abstract->load($this->_placeholder);
    	}
    	
    	return $this;
    }
    
    /**
     * section  
     *
     * section renderer
     *
     * @access  public
     * @param   string   $id
     * @return  Icms
     */
    public function section($id) 
    {
        if (isset($this->_options[ICMS_SECT][$id][$this->_placeholder]) && (
            $this->_options[ICMS_SECT][$id][$this->_placeholder] === ""    or
            $this->_options[ICMS_SECT][$id][$this->_placeholder] === null  or
            $this->_options[ICMS_SECT][$id][$this->_placeholder] === false
            )) {
            return true;  
        } elseif (isset($this->_options[ICMS_SECT][$id][$this->_placeholder])) {
    	    $this->load('section/'.$this->_options[ICMS_SECT][$id][$this->_placeholder]);
    	} elseif (isset($this->_options[ICMS_SECT][$id]["_default"])) {    	    
    	    $this->load('section/'.$this->_options[ICMS_SECT][$id]["_default"]);
    	} elseif (isset($this->_options[ICMS_SECT]["_default"])) {    	    
    	    $this->load('section/'.$this->_options[ICMS_SECT]["_default"]);
    	} elseif (file_exists('section/'.$id.'.php')) {    	    
    	    $this->load('section/'.$id.'.php');
    	} else {    
			$this->error("File for section $id is not exist");
    	}
    }
    
    /**
     * ftp
     *
     * return FTP wrapper
     *
     * @access  public
     * @return  Icms_Ftp
     */
    public function ftp() 
    {
        if (!$this->Ftp) {
            require_once 'Icms/Ftp.php';
            $this->Ftp = new Icms_Ftp();
            $this->Ftp -> connect($this->option(ICMS_FTP));
        }
        return $this->Ftp;
    }
    
    /**
     * scandir
     *
     * function_description
     *
     * @access  public
     * @param   type     $param  param_descr
     * @return  rettype  return
     */
    public function scandir($dir) 
    {
        $dir = ROOT . DS . $dir;
        
        $files = scandir($dir);
        
        unset($files[array_search('.',    $files)]);
        unset($files[array_search('..',   $files)]);
        unset($files[array_search('.svn', $files)]);
        
        unset($files[array_search('readme.txt', $files)]);
        unset($files[array_search('.htaccess', $files)]);
        
        return $files;
    }
    
    /**
     * load
     *
     * load file from root
     *
     * @access  public
     * @param   string   $file
     * @return  Icms_Abstract
     */
    public function load($file) 
    {
        if (file_exists(ROOT . DS . $file)) {
			include ROOT . DS . $file;	
		} else {
			$this->error('File "'.$file.'" not found');
		} 
		return $this;
    }
    
    /**
     * install
     *
     * install application
     *
     * @access  public
     * @return  rettype  return
     */
    public function install($options) 
    {
        if (isset($_POST['options'])) {
            // TODO validator
            $options = array_merge($options, $_POST['options']);
        }
        require_once 'install.php';
    }
    
    /**
     * isAdmin
     *
     * setter for admin mode
     *
     * @access  public
     * @param   bool     $flag
     * @return  Icms|bool
     */
    public function isAdmin($flag = null) 
    {
        if ($flag === null) {
        	return $this->_is_admin;
        } else {
        	$this->_is_admin   = $flag;
        	$_SESSION['admin'] = $flag;
        }
    	return $this;
    }
    
    /**
     * isMain
     *
     * setter for main page
     *
     * @access  public
     * @param   bool     $flag
     * @return  Icms|bool
     */
    public function isMain($flag = null) 
    {
        if ($flag === null) {
        	return $this->_is_main;
        } else {
        	$this->_is_main = $flag;
        }
    	return $this;
    }
    
    /**
     * isDir
     *
     * setter for directory
     *
     * @access  public
     * @param   bool     $flag
     * @return  Icms|bool
     */
    public function isDir($flag = null) 
    {
        if ($flag === null) {
        	return $this->_is_dir;
        } else {
        	$this->_is_dir = $flag;
        }
    	return $this;
    }
    
    /**
     * is404
     *
     * setter for 404 page
     *
     * @access  public
     * @param   bool     $flag
     * @return  Icms|bool
     */
    public function is404($flag = null) 
    {
        if ($flag === null) {
        	return $this->_is_404;
        } else {
        	$this->_is_404 = $flag;
        	if ($flag) {
	            header("HTTP/1.0 404 Not Found");
                header("Status: 404 Not Found");
                
                $this->placeholder('error404.html');
        	}
        }
    	return $this;
    }
    
    /**
     * error
     *
     * print error in debug mode
     *
     * @access  public
     * @param   string     $message
     * @return  rettype  return
     */
    public function error($message) 
    {
        if ($this->_options[ICMS_DEBUG]) {
        	echo $message;
        }
    }
}