<?php
/**
 * Class Icms_Ajax
 *
 * all ajax methods for work with iCMS
 *
 * @access   public
 * @category Icms
 * @package  Icms_Ajax
 * 
 * @access   public
 * @author   dark
 * @version  $Id: Ajax.php Thu Nov 13 11:48:48 EET 2008 11:48:48 dark$
 */
require_once 'Icms/Ajax/Abstract.php';

class Icms_Admin_Ajax extends Icms_Ajax_Abstract
{
    /**
     * toolbarAction
     *
     * create a toolbar
     *
     * @access  public
     * @return  rettype  return
     */
    public function toolbarAction() 
    {
        $include = Icms()->scandir('include');
        $section = Icms()->scandir('section');
        
        require_once 'toolbar.php';
        exit();
    }
        
    /**
     * editAction
     *
     * edit a file
     *
     * @access  public
     * @return  rettype  return
     */
    public function editAction() 
    {
        $file = $_SESSION['edit'];
        $dir  = dirname($file);
        
        if (!file_exists(ROOT . DS . $file)) {            
            if (!file_exists(ROOT . DS . $dir) && !Icms()->Ftp()->mkdir($dir, true)) {
                $this->error('Directory "'.$dir.'" is not writable');
            } else {
                // create empty file
//                Icms()->Ftp()->saveFile($file, '');
//                $fp = fopen(ROOT . DS . $file, 'w');
//                fclose($fp);
            }
        } else {
            if (!Icms()->Ftp()->is_writable($file)) {                
			    $this->error('Can\'t edit file "'.$file .'" don\'t have permissions');
            }
        }
    }
    
    /**
     * saveAction
     *
     * save file
     *
     * @access  public
     * @param   string   $content
     * @return  Icms
     */
    public function saveAction() 
    {
        $file    = $_SESSION['edit'];
        $content = stripslashes(Icms()->Request->get('content'));
        
        try {
            Icms()->Ftp()->saveFile($file, $content);
            $this->message('Page saved');
        } catch (Exception $err) {
            $this->error('Can\'t save: '. $err->getMessage());
        }
    }
    
    /**
     * deleteAction
     *
     * save file
     *
     * @access  public
     * @param   string   $content
     * @return  Icms
     */
    public function deleteAction() 
    {
        $file    = $_SESSION['edit'];

        if (file_exists(ROOT . DS . $file) && Icms()->Ftp()->removeFile($file)) {
			$this->message('Page deleted');
//		} elseif (file_exists(ROOT . DS . $file) && is_dir(ROOT . DS . $file) && @rmdir(ROOT . DS . $file)) {
//			$this->message('Directory deleted');
		} else {
			$this->error('Can\'t delete file (or directory) "'.$file .'" don\'t have permissions');
		}
    }
    
    
    /**
     * phpEditAction
     *
     * load php file for edit by admin
     *
     * @access  public
     * @return  rettype  return
     */
    public function phpEditAction() 
    {
        $file = Icms()->Request->get('file');
        
        $this->message(file_get_contents(ROOT . DS . $file));
    }
    
    /**
     * phpSaveAction
     *
     * load php file for edit by admin
     *
     * @access  public
     * @return  rettype  return
     */
    public function phpSaveAction() 
    {
        $file    = Icms()->Request->get('file');
        $content = stripslashes(Icms()->Request->get('content'));
        
        try {
            Icms()->Ftp()->saveFile($file, $content);
            $this->message('Page saved');
        } catch (Exception $err) {
            $this->error('Can\'t save: '. $err->getMessage());
        }
    }
    
    /**
     * imagesAction
     *
     * return listing of images
     *
     * @access  public
     * @return  rettype  return
     */
    public function imagesAction() 
    {
        $files = $this->_scandir('images');
        
        usort($files, array($this, '_sortDate'));
        $files = array_reverse($files);
        
        echo '<ul>';
        foreach ($files as $file) {
            ?>
            <li><img src="<?php echo $file['url'] ?>" title="<?php echo $file['file'] ?>" alt="<?php echo $file['file'] ?>"/><div name="<?php echo $file['file'] ?>" class="del"></div></li>
            <?php
        }
        echo '</ul>';
        exit();
    }
    
    /**
     * imagesAction
     *
     * return listing of images
     *
     * @access  public
     * @return  rettype  return
     */
    public function filesAction() 
    {
        $files = $this->_scandir();
        
        usort($files, array($this, '_sortDate'));
        $files = array_reverse($files);
        
        echo '<ul>';
        foreach ($files as $file) {
            ?>
            <li><a href="<?php echo $file['url'] ?>" title="<?php echo $file['file'] ?>"><?php echo $file['file'] ?></a><div name="<?php echo $file['file'] ?>" class="del"></div></li>
            <?php
        }
        echo '</ul>';
        exit();
    }
    
    /**
     * fileDelAction
     *
     * remove file from files system
     *
     * @access  public
     * @return  rettype  return
     */
    public function fileDelAction() 
    {
        $file    = Icms()->Request->get('file');
        
//        if (@unlink(ROOT .DS. 'upload' .DS. $file)) {
        if (Icms()->Ftp()->removeFile('upload' .DS. $file)) {
            $this->message('File was removed');
        } else {
            $this->error('Can\'t delete file "'.$file .'"');
        }
    }
    
    /**
     * _sortDate
     *
     * @access  public
     * @param   array
     * @param   array
     * @return  rettype  return
     */
    protected function _sortDate($a, $b) 
    {
        return ($a['date'] < $b['date']) ? -1 : 1;
    }
    
    /**
     * _scandir
     *
     * @access  public
     * @return  rettype  return
     */
    protected function _scandir($type = null) 
    {
        
        $files = Icms()->scandir('upload');
	    
	    switch ($type) {
	        case 'images':
	            $reg = '/\.(ico|gif|jpg|jpeg|png|bmp)$/i';
	            break;
	        case 'archive':
	            $reg = '/\.(zip|rar|tgz|gz|7z)$/i';
	            break;
	        case 'docs':
	            $reg = '/\.(pdf|doc|txt)$/i';
	            break;
	        default:
	            $reg = '/.*\..*/i';
	            break;
	    }
	    
	    $res = array();
	     
        foreach ($files as $file) {
            
            switch (true) {
                case ($file == '.'):
                case ($file == '..'):
                    break;
                case (preg_match($reg, $file)):
                    $info['url']  = DOMAIN . Icms()->option(ICMS_PATH) . 'upload/' . $file;
                    $info['file'] = $file;
                    $info['size'] = filesize(ROOT .DS. 'upload' .DS. $file);
                    $info['date'] = filemtime(ROOT .DS. 'upload' .DS. $file);
                    $res[] = $info;
                    break;
            }
        }
        return $res;
    }
}