<?php
/**
 * Class Icms_Ftp
 *
 * class for work with CMS over FTP
 *
 * @access   public
 * @category Icms_Ftp
 * @package  Icms_Ftp
 * 
 * @access   public
 * @author   dark
 * @version  $Id: Ftp.php Wed Jan 14 15:03:13 EET 2009 dark$
 */
class Icms_Ftp 
{
    protected $handle;

    /**
     * connect
     *
     * connect to our FTP server
     *
     * @access  public
     * @param   string   $host
     * @param   integer  $port
     * @param   string   $login
     * @param   string   $password
     * @param   string   $dir
     * @return  return void
     */
    public function connect($options) 
    {
        $options = array_merge(array('port' => 21, 'user' => 'anonymous', 'pass' => '', 'path' => ''), $options);
        
        $this->options = $options;
        $this->handle  = @ftp_connect($options['host'], $options['port']);
        
        if (!$this->handle) {
            throw new Exception('Error connection to "'.$options['host'].':'.$options['port'].'"');
        } elseif (@ftp_login($this->handle, $options['user'], $options['pass'])) {
            
            if ($options['path'] == '' or $this->cd($options['path'])) {
                if ($options['active']) {
                    $this->setActive();
                } else {
                    $this->setPassive();
                }
                return true;
            } else {
                throw new Exception('Wrong directory name');
            }
        } else {
            throw new Exception('Wrong login or password');
        }
    }
    
    /**
     * close
     *
     * close FTP connection
     *
     * @access  public
     * @return  void
     */
    public function close() 
    {
        return @ftp_close($this->handle);
    }
    
    /**
     * This changes the currently used directory. You can use either an absolute
     * directory-path (e.g. "/home/blah") or a relative one (e.g. "../test").
     *
     * @param string $dir The directory to go to.
     *
     * @access public
     * @return mixed True on success
     */
    public function cd($dir)
    {
        $erg = @ftp_chdir($this->handle, $dir);

        if ($erg !== true) {
            return false;
        } else {
            return true;
        }
    }
    
    /**
     * Show's you the actual path on the server
     * This function questions the ftp-handle for the actual selected path and
     * returns it.
     *
     * @access public
     * @return mixed The actual path
     */
    public function pwd()
    {
        $res = @ftp_pwd($this->handle);
        if (!$res) {
            throw new Exception("Could not determine the actual path.");
        } else {
            return $res;
        }
    }
    
    /**
     * This works similar to the mkdir-command on your local machine. You can either
     * give it an absolute or relative path. The relative path will be completed
     * with the actual selected server-path. (see: pwd())
     *
     * @param string $dir       Absolute or relative dir-path
     * @param bool   $recursive (optional) Create all needed directories
     *
     * @access public
     * @return mixed True on success
     */
    public function mkdir($dir, $recursive = false)
    {
        $dir     = $this->_constructPath($dir);
        $savedir = $this->pwd();
        $e = $this->cd($dir);
        if ($e === true) {
            $this->cd($savedir);
            return true;
        }
        $this->cd($savedir);
        if ($recursive === false) {
            $res = @ftp_mkdir($this->handle, $dir);
            if (!$res) {
                throw new Exception("Creation of '$dir' failed");
            } else {
                return true;
            }
        } else {
            // do not look at the first character, as $dir is absolute,
            // it will always be a /
            if (strpos(substr($dir, 1), '/') === false) {
                return $this->mkdir($dir, false);
            }
            if (substr($dir, -1) == '/') {
                $dir = substr($dir, 0, -1);
            }
            $parent = substr($dir, 0, strrpos($dir, '/'));
            $res    = $this->mkdir($parent, true);
            if ($res === true) {
                $res = $this->mkdir($dir, false);
            }
            if ($res !== true) {
                return $res;
            }
            return true;
        }
    }
    
    /**
     * Rebuild the path, if given relative
     *
     * This method will make a relative path absolute by prepending the current
     * remote directory in front of it.
     *
     * @param string $path The path to check and construct
     *
     * @access private
     * @return string The build path
     */
    private function _constructPath($path)
    {
        if ((substr($path, 0, 1) != '/') && (substr($path, 0, 2) != './')) {
            $actual_dir = @ftp_pwd($this->handle);
            if (substr($actual_dir, -1) != '/') {
                $actual_dir .= '/';
            }
            $path = $actual_dir.$path;
        }
        return $path;
    }
    
    /**
     * Set the transfer-method to passive mode
     *
     * @access public
     * @return void
     */
    public function setPassive()
    {
        @ftp_pasv($this->handle, true);
    }

    /**
     * Set the transfer-method to active mode
     *
     * @access public
     * @return void
     */
    public function setActive()
    {
        @ftp_pasv($this->handle, false);
    }
    
    /**
     * saveFile
     *
     * save file via FTP
     *
     * @access  public
     * @param   string   $file
     * @return  rettype  return
     */
    public function saveFile($file, $content) 
    {
        $temp = tempnam('/tmp', 'icms');
        
        $fp   = fopen($temp, 'w');
                fwrite($fp, $content);
                fclose($fp);
        $res  = @ftp_put($this->handle, $file, $temp, FTP_BINARY);
        
        @unlink($temp);

        if (!$res) {
            throw new Exception("don't have permission for upload \"$file\" to FTP");
        }
        
//        $user = $this->options['user'];
//        $pass = $this->options['pass'];
//        $host = $this->options['host'];
//        $port = $this->options['port'];
//        $path = $this->options['path'];
//        $opts    = array('ftp' => array('overwrite' => true));
//        $context = stream_context_create($opts);
//
//        $handle = @fopen("ftp://$user:$pass@$host:$port/$path/$file", "w", false, $context);
//        
//        if (fwrite($handle, $content) === false) {
//            throw new Exception("Cannot write to file ($file)");
//        }
        
        return true;
    }
    
    /**
     * is_writable
     *
     * save file via FTP
     *
     * @access  public
     * @param   string   $file
     * @return  rettype  return
     */
    public function is_writable($file) 
    {
//        $user = $this->options['user'];
//        $pass = $this->options['pass'];
//        $host = $this->options['host'];
//        $port = $this->options['port'];
//        $path = $this->options['path'];
        
//        echo "ftp://$user:$pass@$host:$port/$path/$file";
//        return is_writable("ftp://$user:$pass@$host:$port/$path/$file");
        return true;
    }
    
    /**
     * uploadFile
     *
     * upload file to FTP
     *
     * @access  public
     * @param   string   $local_file
     * @param   string   $remote_file
     * @return  rettype  return
     */
    public function uploadFile($local_file, $remote_file) 
    {
        $res  = ftp_put($this->handle, $remote_file, $local_file, FTP_BINARY);
        
        if (!$res) {
            throw new Exception("FTP server return error");
            return false;
        }
        return true;
    }
    
    /**
     * This will remove a file
     *
     * @param string $file The file to delete
     *
     * @access private
     * @return mixed True on successs
     */
    public function removeFile($file)
    {
//        if (substr($file, 0, 1) != "/") {
//            $actual_dir = @ftp_pwd($this->handle);
//            if (substr($actual_dir, (strlen($actual_dir) - 2), 1) != "/") {
//                $actual_dir .= "/";
//            }
//            $file = $actual_dir.$file;
//        }
        $res = @ftp_delete($this->handle, $file);
        
        if (!$res) {
            throw new Exception("Could not delete file '$file'.");
        } else {
            return true;
        }
    }
    
}