<?php
/**
 * Class Icms_Request
 *
 * work with request
 *
 * @access   public
 * @category Icms
 * @package  Icms_Request
 * 
 * @access   public
 * @author   dark
 * @version  $Id: Request.php Tue Nov 11 16:54:29 EET 2008 16:54:29 dark$
 */
class Icms_Request 
{
    /**
     * REQUEST_URI
     * @var string;
     */
    protected $_requestUri;
    
    public $uri;
    public $path;
    public $file;
    public $info;
    
    /**
     * Constructor of Icms_Request
     *
     * @access  public
     */
    function __construct() 
    {
        $this->setRequestUri()
        	 ->parseUri();
    }
    
    /**
     * get
     *
     * return params from request
     *
     * @access  public
     * @param   string     $key     param name
     * @param   mixed      $default value if not exist
     * @return  mixed
     */
    public function get($key, $default = null) 
    {
        switch (true) {
            case isset($_GET[$key]):
                return $_GET[$key];
            case isset($_POST[$key]):
                return $_POST[$key];
            case isset($_COOKIE[$key]):
                return $_COOKIE[$key];
            case isset($_SERVER[$key]):
                return $_SERVER[$key];
            case isset($_ENV[$key]):
                return $_ENV[$key];
            default:
                return $default;
        }
    }
    
    /**
     * parseUri
     *
     * parse URI
     *
     * @access  public
     * @return  Core_Request
     */
    public function parseUri() 
    {
    	if (!$this->_requestUri) $this->setRequestUri();
    	
    	if (strpos($this->_requestUri, Icms()->option(ICMS_PATH)) === 0) {
    		$this->_requestUri = substr($this->_requestUri, strlen(Icms()->option(ICMS_PATH)));
    	}
    	
    	
    	if (strpos($this->_requestUri, 'application') !== false or 
    	    strpos($this->_requestUri, 'include') !== false or
    	    strpos($this->_requestUri, 'section') !== false
    	   ) {
    	    // permission denied
    	    return $this;
    	}
    	
    	$this->uri  = $this->_requestUri;
    	$this->path = dirname($this->_requestUri);
    	$this->file = basename($this->_requestUri);
    	$this->info = pathinfo($this->_requestUri);

        return $this;
    }
    
    
    /**
     * Set the REQUEST_URI on which the instance operates
     *
     * If no request URI is passed, uses the value in $_SERVER['REQUEST_URI'],
     * $_SERVER['HTTP_X_REWRITE_URL'], or $_SERVER['ORIG_PATH_INFO'] + $_SERVER['QUERY_STRING'].
     *
     * @see Zend_Controller_Request_Http
     * @param string $requestUri
     * @return string
     */
    public function getRequestUri()
    {
        if (!$this->_requestUri) $this->setRequestUri();
        
        return $this->_requestUri;
    }
    
    /**
     * Set the REQUEST_URI on which the instance operates
     *
     * If no request URI is passed, uses the value in $_SERVER['REQUEST_URI'],
     * $_SERVER['HTTP_X_REWRITE_URL'], or $_SERVER['ORIG_PATH_INFO'] + $_SERVER['QUERY_STRING'].
     *
     * @see Zend_Controller_Request_Http
     * @param string $requestUri
     * @return Icms_Request
     */
    public function setRequestUri($requestUri = null)
    {
        if ($requestUri === null) {
            if (isset($_SERVER['HTTP_X_REWRITE_URL'])) { // check this first so IIS will catch
                $requestUri = $_SERVER['HTTP_X_REWRITE_URL'];
            } elseif (isset($_SERVER['REQUEST_URI'])) {
                $requestUri = $_SERVER['REQUEST_URI'];
            } elseif (isset($_SERVER['ORIG_PATH_INFO'])) { // IIS 5.0, PHP as CGI
                $requestUri = $_SERVER['ORIG_PATH_INFO'];
                if (!empty($_SERVER['QUERY_STRING'])) {
                    $requestUri .= '?' . $_SERVER['QUERY_STRING'];
                }
            } else {
                return $this;
            }
        } elseif (!is_string($requestUri)) {
            return $this;
        }

        $this->_requestUri = $requestUri;
        return $this;
    }
    
    /**
     * Return the value of the given HTTP header. Pass the header name as the
     * plain, HTTP-specified header name. Ex.: Ask for 'Accept' to get the
     * Accept header, 'Accept-Encoding' to get the Accept-Encoding header.
     *
     * @see    Zend_Controller_Request_Http
     * @param  string HTTP header name
     * @return string|false HTTP header value, or false if not found
     * @throws Exception
     */
    public function getHeader ($header)
    {
        if (empty($header)) {
            throw new Exception('An HTTP header name is required');
        }
        // Try to get it from the $_SERVER array first
        $temp = 'HTTP_' . strtoupper(str_replace('-', '_', $header));
        if (! empty($_SERVER[$temp])) {
            return $_SERVER[$temp];
        }
        // This seems to be the only way to get the Authorization header on
        // Apache
        if (function_exists('apache_request_headers')) {
            $headers = apache_request_headers();
            if (! empty($headers[$header])) {
                return $headers[$header];
            }
        }
        return false;
    }

    /**
     * Is the request a Javascript XMLHttpRequest?
     *
     * Should work with Prototype/Script.aculo.us, possibly others.
     *
     * @return boolean
     */
    public function isXmlHttpRequest ()
    {
        return ($this->getHeader('X_REQUESTED_WITH') == 'XMLHttpRequest');
    }
    
}